<?php
class Quivers_Integration_Model_Observer
{
    public function taxUpdate($observer)
    {
        $marketPlace = Mage::getStoreConfig('quivers_integration/settings/quivers_marketplace_id');
        $quiversSandbox = Mage::getStoreConfig('quivers_integration/settings/quivers_sandbox_enabled');
        if($quiversSandbox) {
            $taxUrl = 'https://api.quiverstest.com/v1/CustomerOrders/Validate';
        }
        else {
            $taxUrl = 'https://api.quivers.com/v1/CustomerOrders/Validate';   
        }
        if(!$marketPlace) {
            return;
        }

        $quote = Mage::getSingleton('checkout/session')->getQuote();
        $shippingMethods = array();
        $addresses = $quote->getAllShippingAddresses();
        
        $shippingFeesTotal =  0.0;
        $shippingFeePerItem = 0.0;

        foreach ($addresses as $address) {
            $id = $address->getCustomerAddressId();
            // Mage::log($address->toJson());
            // Mage::log(get_class_methods($address));

            $items = array();
            $productMapping = array();

            // if no item selected then no need to proceed further.
            if(count($quote->getAllItems()) <= 0) {
                // Mage::log(count($quote->getAllItems()));
                return $this;
            }

            // $shipping_address = $quote->getShippingAddress();
            // Mage::log("seee all shipping address");
            // Mage::log(count($quote->getAllShippingAddresses()));
            // if(!$shipping_address) {
            //     return $this;
            // }

            $country = $address->getCountryId();

            if(!$country) {
                return $this;
            }

            if(!$address->getRegionCode()) {
                return $this;
            }

            $shippingMethod = $address->getShippingMethod();
            
            if($shippingMethod) {
                $shippingRateData = $address->getShippingRatesCollection()->getData();

                if(count($shippingRateData)>0) {
                    foreach ($shippingRateData as $rate) {
                        if(array_key_exists('price', $rate)) {
                            $shippingFeesTotal += $rate["price"];
                        }
                    }
                }

                $totalItems = 0;
                foreach ($address->getAllVisibleItems() as $quoteItem) {
                    // $itemProduct = $quoteItem->getProduct();
                    // $unitPrice = (float)$itemProduct->getPrice();
                    $parent = $quoteItem->getParentItem();
                    if(!$parent) {
                        $totalItems = $totalItems +1;
                    }
                }
                if($totalItems ==0) {
                    return $this;
                }
                $shippingFeePerItem = (float)$shippingFeesTotal/$totalItems;

            }
            foreach ($address->getAllVisibleItems() as $quoteItem) {
                $parent = $quoteItem->getParentItem();
                if($parent) {
                    continue;
                }
                // Mage::log($quoteItem->toJson());
                $itemData = $quoteItem->getData();
                $itemProduct = $quoteItem->getProduct();
                $unitPrice = (float)$itemProduct->getPrice();
                // skip virtual products
                // Mage::log('unit price');
                // Mage::log("unit price".$unitPrice);

                $itemId = (int)$itemProduct->getId();

                $productMapping[$itemId] = $quoteItem;

                $itemName = $itemProduct->getName();
                $itemVariantName = $itemProduct->getSku();
                $options = array();
                $options = $quoteItem->getProduct()->getTypeInstance(true)->getOrderOptions($quoteItem->getProduct());
                $options_arr = array();
                if (isset($options['options']) && !empty($options['options'])) {
                    foreach ($options['options'] as $option) {
                        if(array_key_exists('label', $option)) {
                            $optionTitle = $option['label'];
                        }
                        if(array_key_exists('option_id', $option)) {
                            $optionId = $option['option_id'];
                        }
                        if(array_key_exists('value', $option)) {
                            $optionValue = $option['value'];
                        }
                        
                    array_push($options_arr, array(
                        "name" => $optionTitle,
                        "value" => array("name" => $optionValue)
                    ));

                    }
                }

                if(array_key_exists('base_price', $itemData)) {
                    $unitPrice = $itemData['base_price'];
                }

                $totalDiscount = $quoteItem->getDiscountAmount();

                $discounts = array();
                if($totalDiscount > 0 ) {
                    array_push($discounts,  array(
                        "name"=> "Total Discount",
                        "amount"=> -$totalDiscount,
                    ));
                }

                $shippingFees = array();
                if($shippingFeePerItem > 0 ) {
                    $shippingFees = array(
                        "name" => $shippingMethod,
                        "amount" => $shippingFeePerItem
                    );
                }

                if(count($shippingFees) > 0 ) {
                    $shippingFees = array($shippingFees);
                }

                array_push($items, array(
                "product" => array(
                    "name" => $itemName,
                    "id" => $itemId,
                    "variant" => array(
                        "name" => $itemVariantName,
                        "options" => $options_arr
                    )
                ),

                "quantity" => $quoteItem->getQty(),
                "pricing" => array(
                        "unitPrice" => (float)$unitPrice,
                        "shippingFees" => $shippingFees,
                        "discounts"=> $discounts
                    )
                ));

            }
            $taxData = array(
                "marketplaceId" => $marketPlace,
                "shippingAddress" => array(
                    "phoneNumber" => $address->getTelephone(),
                    "line1" => $address->getStreet1(),
                    "line2" => $address->getStreet2(),
                    "city" => $address->getCity(),
                    "postCode" => $address->getPostcode(),
                    "country" => $country,
                    "region" => $address->getRegionCode()
                ),
                "items" => $items
            );


            // Mage::log($taxData);


            $ch = curl_init( $taxUrl );
            # Setup request to send json via POST.
            $payload = json_encode($taxData);
            curl_setopt( $ch, CURLOPT_POSTFIELDS, $payload );
            curl_setopt( $ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            # Return response instead of printing.
            curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
            # Send request.
            $result = curl_exec($ch);
            $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);
            
            if($httpcode==200) {
                $responseDecoded = json_decode($result, true);
                $totalTax = $responseDecoded["result"]["totals"]["taxes"]["additional"];
                $subtotal = $responseDecoded["result"]["totals"]["subtotal"];
                $totalAmount = $responseDecoded["result"]["totals"]["total"];
                foreach ($responseDecoded["result"]["items"] as $responseItems) {
                    $productId = $responseItems['productId'];
                    $itemQuantity = $responseItems['quantity'];
                    $itemSubotal = $responseItems['pricing']['subtotal'] - $shippingFeePerItem;
                    $itemTotal = $responseItems['pricing']['total'] - $shippingFeePerItem;
                    $itemUnitPrice = $responseItems['pricing']['unitPrice'];

                    $quoteItem = $productMapping[$productId];
                    $quoteItemData = $quoteItem->getData();
                    $appliedTaxes = $responseItems['pricing']['taxes'];
                    if(count($responseItems['pricing']['taxes']) <= 0) {
                        continue;
                    }
                    $quoteItemData['price_incl_tax'] = $itemTotal/$itemQuantity;
                    $quoteItemData['base_price_incl_tax'] = $itemTotal/$itemQuantity;
                    $quoteItemData['tax_amount'] = $appliedTaxes[0]['amount'];
                    $quoteItemData['base_tax_amount'] = $appliedTaxes[0]['amount'];
                    $quoteItemData['tax_percent'] = $appliedTaxes[0]['rate']*100;
                    $quoteItemData['row_total_incl_tax'] = $itemTotal;
                    $quoteItem->setData($quoteItemData);

                    // $quoteItem->collectTotals();
                }
                // Mage::log(get_class_methods($address));
                // Mage::log($address->toJson());
                $addressData = $address->getData();
                $addressData['subtotal'] = $subtotal;
                $addressData['base_subtotal'] = $subtotal;
                $addressData['tax_amount'] = $totalTax;
                $addressData['base_tax_amount'] = $totalTax;
                $addressData['grand_total'] = $totalAmount;
                $addressData['base_grand_total'] = $totalAmount;
                $addressData['subtotal_incl_tax'] = $subtotal + $totalTax;
                $addressData['base_subtotal_total_incl_tax'] = $subtotal + $totalTax;
                $address->setGrandTotal($totalAmount);
                $address->setBaseGrandTotal($totalAmount);
                $address->setData($addressData);
                
            }
        }
    }
        
}